@@ -4,7 +4,7 @@
 
 ;; Filename: coterm.el
 ;; Author: jakanakaevangeli <jakanakaevangeli@chiru.no>
-;; Version: 1.2
+;; Version: 1.3
 ;; Keywords: processes
 ;; Package-Requires: ((emacs "26.1"))
 ;; URL: https://repo.or.cz/emacs-coterm.git
@@ -54,11 +54,11 @@
 ;; automatic char-mode, char-mode enabled and char-mode disabled.
 ;;
 ;;
-;; Installation:
+;;;; Installation:
 ;;
 ;; To install coterm, type M-x package-install RET coterm RET
 ;;
-;; It is best to add the following elisp snippet to you Emacs init file, to
+;; It is best to add the following elisp snippet to your Emacs init file, to
 ;; enable `coterm-mode' automatically on startup:
 ;;
 ;;   (coterm-mode)
@@ -72,7 +72,7 @@
 ;;   ;;   (setq-default comint-process-echoes t)
 ;;
 ;;
-;; Differences from M-x term:
+;;;; Differences from M-x term:
 ;;
 ;; coterm is written as an upgrade to comint.  For existing comint users, the
 ;; behaviour of comint doesn't change with coterm enabled except for the added
@@ -84,7 +84,7 @@
 ;; need to manually enable and disable char mode.
 ;;
 ;;
-;; Some common probles:
+;;;; Some common probles:
 ;;
 ;; If some TUI programs misbehave, try checking your TERM environment variable
 ;; with 'echo $TERM' in your coterm enabled M-x shell.  It should normally be
@@ -107,7 +107,7 @@
 ;; option off or to use manual char mode.
 ;;
 ;;
-;; Bugs, suggestions and patches can be sent to
+;;;; Bugs, suggestions and patches can be sent to
 ;;
 ;;    bugs-doseganje (at) groups.io
 ;;
@@ -419,7 +419,7 @@ that char mode is maintained even if the user presses \"/\",
              "byte [0-9]+\\|"
              "lines [0-9]+-[0-9]+\\|"
              "100%\\|"
-             "\\(?:.* \\)?" "[0-9]?[0-9]%\\|"
+             "\\(?:.* \\)?" "\\(:?[0-9]?[0-9]\\|100\\)" "%\\|"
              ".*(press h for help or q to quit)\\|"
              ".*(press RETURN)"
              "\\)\\'"))
@@ -519,7 +519,7 @@ is the process mark."
 ;; This is essentially a re-implementation of term.el's terminal emulation.  I
 ;; could have simply reused functions from term.el but that would have been
 ;; unsatisfactory in my opinion.  That is mostly due to the fact that term.el's
-;; terminal emulation inserts a lot of redundant trailing whitespace end empty
+;; terminal emulation inserts a lot of redundant trailing whitespace and empty
 ;; lines, which I believe is very distracting for ordinary comint usage.
 ;;
 ;; Terminal emulation is coordinate based, for example, "move cursor to row 11
@@ -527,7 +527,7 @@ is the process mark."
 ;; buffer because the specified line is currently too short or there aren't
 ;; enough lines in the buffer.  term.el automatically inserts empty lines and
 ;; spaces in order to move point to a specified coordinate position.  This
-;; results in trailing whitespace.
+;; often results in trailing whitespace.
 ;;
 ;; coterm takes a different approach.  Instead of moving point, the current
 ;; terminal cursor coordinates are kept in the variables `coterm--t-row' and
@@ -557,7 +557,7 @@ is the process mark."
 ;; terminal emulation isn't fully accurate for long lines.  Up to now, "less"
 ;; was the only program I've encountered that relies on accurate line wrapping,
 ;; so a workaround aimed at "less" specifically was implemented (search for the
-;; term "less" in the function `coterm--t-emulate-terminal'.
+;; term "less" in the function `coterm--t-emulate-terminal').
 
 (defconst coterm--t-control-seq-regexp
   ;; Differences from `term-control-seq-regexp':
@@ -568,12 +568,13 @@ is the process mark."
   ;; Removed: \032 (\C-z)
   ;; Added: OSC sequence \e] ... ; ... \e\\ (or \a)
   ;; Added: sequences \e= and \e>
+  ;; Added: Invalid sequence \e\e, used by package `bash-completion'
   (concat
    ;; A control character,
    "\\(?:[\n\000\007\t\b\016\017]\\|\r\n?\\|"
    ;; a C1 escape coded character (see [ECMA-48] section 5.3 "Elements
    ;; of the C1 set"),
-   "\e\\(?:[DM78c=>]\\|"
+   "\e\\(?:[DM78c=>\e]\\|"
    ;; Emacs specific control sequence from term.el.  In coterm, we simply
    ;; ignore them.
    "AnSiT[^\n]+\n\\|"
@@ -869,7 +870,7 @@ buffer and the scrolling region must cover the whole screen."
     (setq coterm--t-col column)))
 
 (defun coterm--t-adjust-from-pmark (pos)
-  "Point `coterm--t-row' and `coterm--t-col' POS."
+  "Make `coterm--t-row' and `coterm--t-col' point to POS."
   (coterm--t-normalize-home-offset)
   (goto-char pos)
   (setq coterm--t-col (current-column))
@@ -915,7 +916,11 @@ buffer and the scrolling region must cover the whole screen."
          (car-or-1 ()
            `(max 1 (car ctl-params)))
          (cadr-or-0 ()
-           `(or (cadr ctl-params) 0)))
+           `(or (cadr ctl-params) 0))
+         (at-eob ()
+           `(and coterm--t-pmark-in-sync
+                 (= pmark (point-max))
+                 (not (coterm--t-scroll-by-deletion-p)))))
 
       (if (not (and string
                     (setq buf (process-buffer process))
@@ -956,9 +961,7 @@ buffer and the scrolling region must cover the whole screen."
                  ;; when the process just outputs text at eob without any
                  ;; control sequences, we will end up inserting the whole
                  ;; string without a single call to `substring'.
-                 (if (and coterm--t-pmark-in-sync
-                          (= pmark (point-max))
-                          (not (coterm--t-scroll-by-deletion-p)))
+                 (if (at-eob)
                      (progn (pass-through)
                             (cl-incf will-insert-newlines))
                    (ins)
@@ -970,10 +973,7 @@ buffer and the scrolling region must cover the whole screen."
                      (setq coterm--t-col 0)
                      (dirty))
                 ;; TAB (terminfo: ht)
-                ((and ?\t
-                      (guard coterm--t-pmark-in-sync)
-                      (guard (= pmark (point-max)))
-                      (guard (not (coterm--t-scroll-by-deletion-p))))
+                ((and ?\t (guard (at-eob)))
                  ;; Insert a TAB as is, if at eob
                  (pass-through))
                 (?\t
